# -*- coding: utf-8 -*-
"""Production servers for python Web applications

Example usage (for cherry.py server):

    >>> wsgi_app = "my crazy application wsgi file"
    >>> cherrypy_factory(
    ...     servers=3,
    ...     port_list=[8081, 8082, 8083],
    ...     application=wsgi_app,
    ...     thread_pool=10
    ... )
    >>>


TODO - More robust logging mechanism (use paste).
     - Auto reload on application instance change.
"""

__version__ = '0.1'

__author__ = 'pav'

__date__ = '01.01.2014'

__license__ = "GNU General Public License (GPL), Version 3"

__all__ = ['cherrypy_factory']


import cherrypy


def cherrypy_factory(servers, port_list, application, thread_pool=30):
    """Creates production multi-thread server instances for any
    WSGI-compiliant web application

    Arguments:

      - servers (int)       : The number of application servers
      - port_list (list)    : The available ports for every server
      - application(object) : Any WSGI-compiliant application instance

    Raises:
      - AssertionError: If number of ports < number of application servers.
    """

    assert len(range(0, servers)) <= len(port_list), "Ports provided cant cover number of servers."

    server_list = {}

    cherrypy.tree.graft(application, "/")   # mount the application
    cherrypy.server.unsubscribe()    # Unsubscribe the default server

    for server_uid, server_port in zip(range(0, servers), port_list):
        p_server = 'server_{}'.format(server_uid)
        server_list[p_server] = cherrypy._cpserver.Server()
        server_list[p_server].socket_host = "0.0.0.0"
        server_list[p_server].socket_port = server_port
        server_list[p_server].thread_pool = thread_pool
        server_list[p_server].subscribe()

    cherrypy.engine.start()
    cherrypy.engine.block()